@@ -23,6 +23,8 @@ from twisted.web import iweb, http, error
 
 from twisted.web.test.requesthelper import DummyChannel, DummyRequest
 from twisted.web.static import Data
+from twisted.logger import globalLogPublisher, LogLevel
+from twisted.test.proto_helpers import EventLoggingObserver
 
 
 class ResourceTests(unittest.TestCase):
@@ -499,6 +501,14 @@ class RequestTests(unittest.TestCase):
             verifyObject(iweb.IRequest, server.Request(DummyChannel(), True)))
 
 
+    def test_hashable(self):
+        """
+        L{server.Request} instances are hashable, thus can be put in a mapping.
+        """
+        request = server.Request(DummyChannel(), True)
+        hash(request)
+
+
     def testChildLink(self):
         request = server.Request(DummyChannel(), 1)
         request.gotLength(0)
@@ -593,6 +603,11 @@ class RequestTests(unittest.TestCase):
         to C{False} does not write out the failure, but give a generic error
         message.
         """
+        logObserver = EventLoggingObserver.createWithCleanup(
+            self,
+            globalLogPublisher
+        )
+
         d = DummyChannel()
         request = server.Request(d, 1)
         request.site = server.Site(resource.Resource())
@@ -604,6 +619,12 @@ class RequestTests(unittest.TestCase):
         self.assertIn(
             b"Processing Failed", request.transport.written.getvalue()
         )
+        self.assertEquals(1, len(logObserver))
+
+        event = logObserver[0]
+        f = event["log_failure"]
+        self.assertIsInstance(f.value, Exception)
+        self.assertEquals(f.getErrorMessage(), "Oh no!")
 
         # Since we didn't "handle" the exception, flush it to prevent a test
         # failure
@@ -615,6 +636,11 @@ class RequestTests(unittest.TestCase):
         L{Request.processingFailed} when the site has C{displayTracebacks} set
         to C{True} writes out the failure.
         """
+        logObserver = EventLoggingObserver.createWithCleanup(
+            self,
+            globalLogPublisher
+        )
+
         d = DummyChannel()
         request = server.Request(d, 1)
         request.site = server.Site(resource.Resource())
@@ -624,6 +650,10 @@ class RequestTests(unittest.TestCase):
 
         self.assertIn(b"Oh no!", request.transport.written.getvalue())
 
+        event = logObserver[0]
+        f = event["log_failure"]
+        self.assertIsInstance(f.value, Exception)
+        self.assertEquals(f.getErrorMessage(), "Oh no!")
         # Since we didn't "handle" the exception, flush it to prevent a test
         # failure
         self.assertEqual(1, len(self.flushLoggedErrors()))
@@ -635,6 +665,11 @@ class RequestTests(unittest.TestCase):
         to C{True} writes out the failure, making UTF-8 items into HTML
         entities.
         """
+        logObserver = EventLoggingObserver.createWithCleanup(
+            self,
+            globalLogPublisher
+        )
+
         d = DummyChannel()
         request = server.Request(d, 1)
         request.site = server.Site(resource.Resource())
@@ -650,6 +685,9 @@ class RequestTests(unittest.TestCase):
         # uses a default encodig of cp437 which does not support u"\u2603".
         self.flushLoggedErrors(UnicodeError)
 
+        event = logObserver[0]
+        f = event["log_failure"]
+        self.assertIsInstance(f.value, Exception)
         # Since we didn't "handle" the exception, flush it to prevent a test
         # failure
         self.assertEqual(1, len(self.flushLoggedErrors()))
@@ -952,22 +990,47 @@ class GzipEncoderTests(unittest.TestCase):
 
 
 class RootResource(resource.Resource):
-    isLeaf=0
+    isLeaf = 0
+
     def getChildWithDefault(self, name, request):
         request.rememberRootURL()
         return resource.Resource.getChildWithDefault(self, name, request)
+
+
     def render(self, request):
         return ''
 
+
+
 class RememberURLTests(unittest.TestCase):
+    """
+    Tests for L{server.Site}'s root request URL calculation.
+    """
+
     def createServer(self, r):
+        """
+        Create a L{server.Site} bound to a L{DummyChannel} and the
+        given resource as its root.
+
+        @param r: The root resource.
+        @type r: L{resource.Resource}
+
+        @return: The channel to which the site is bound.
+        @rtype: L{DummyChannel}
+        """
         chan = DummyChannel()
         chan.site = server.Site(r)
         return chan
 
+
     def testSimple(self):
+        """
+        The path component of the root URL of a L{server.Site} whose
+        root resource is below C{/} is that resource's path, and the
+        netloc component is the L{site.Server}'s own host and port.
+        """
         r = resource.Resource()
-        r.isLeaf=0
+        r.isLeaf = 0
         rr = RootResource()
         r.putChild(b'foo', rr)
         rr.putChild(b'', rr)
@@ -978,9 +1041,16 @@ class RememberURLTests(unittest.TestCase):
             request.setHost(b'example.com', 81)
             request.gotLength(0)
             request.requestReceived(b'GET', url, b'HTTP/1.0')
-            self.assertEqual(request.getRootURL(), b"http://example.com/foo")
+            self.assertEqual(request.getRootURL(),
+                             b"http://example.com:81/foo")
+
 
     def testRoot(self):
+        """
+        The path component of the root URL of a L{server.Site} whose
+        root resource is at C{/} is C{/}, and the netloc component is
+        the L{site.Server}'s own host and port.
+        """
         rr = RootResource()
         rr.putChild(b'', rr)
         rr.putChild(b'bar', resource.Resource())
@@ -990,7 +1060,9 @@ class RememberURLTests(unittest.TestCase):
             request.setHost(b'example.com', 81)
             request.gotLength(0)
             request.requestReceived(b'GET', url, b'HTTP/1.0')
-            self.assertEqual(request.getRootURL(), b"http://example.com/")
+            self.assertEqual(request.getRootURL(),
+                             b"http://example.com:81/")
+
 
 
 class NewRenderResource(resource.Resource):
@@ -1078,6 +1150,11 @@ class NewRenderTests(unittest.TestCase):
         self.assertEqual([b'GET', b'HEAD', b'HEH'], allowed)
 
     def testImplicitHead(self):
+        logObserver = EventLoggingObserver.createWithCleanup(
+            self,
+            globalLogPublisher
+        )
+
         req = self._getReq()
         req.requestReceived(b'HEAD', b'/newrender', b'HTTP/1.0')
         self.assertEqual(req.code, 200)
@@ -1085,12 +1162,21 @@ class NewRenderTests(unittest.TestCase):
             -1, req.transport.written.getvalue().find(b'hi hi')
         )
 
+        self.assertEquals(1, len(logObserver))
+        event = logObserver[0]
+        self.assertEquals(event["log_level"], LogLevel.info)
+
 
     def test_unsupportedHead(self):
         """
         HEAD requests against resource that only claim support for GET
         should not include a body in the response.
         """
+        logObserver = EventLoggingObserver.createWithCleanup(
+            self,
+            globalLogPublisher
+        )
+
         resource = HeadlessResource()
         req = self._getReq(resource)
         req.requestReceived(b"HEAD", b"/newrender", b"HTTP/1.0")
@@ -1098,6 +1184,8 @@ class NewRenderTests(unittest.TestCase):
         self.assertEqual(req.code, 200)
         self.assertEqual(body, b'')
 
+        self.assertEquals(2, len(logObserver))
+
 
     def test_noBytesResult(self):
         """
